package rainmelt;

import java.io.IOException;
import java.lang.reflect.Type;

import net.tootallnate.websocket.Base64;

import org.java_websocket.WebSocket;
import org.java_websocket.handshake.ClientHandshake;

import rainmelt.api.API;
import rainmelt.api.APITable;
import rainmelt.command.Command;
import rainmelt.command.CommandTable;
import rainmelt.fileapi.FileAPI;
import rainmelt.fileapi.FileAPITable;
import rainmelt.packet.APIPacket;
import rainmelt.packet.CommandPacket;
import rainmelt.packet.FileAPIPacket;
import rainmelt.packet.Packet;
import rainmelt.packet.ResultPacket;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;

public final class Handler {
	
	private final WebSocket webSocket;
	private final ClientHandshake handshake;
	
	private final Gson gson;
	
	private final FileSystem fileSystem = new FileSystem();
	private Account account = AccountManager.INSTANCE.get(AccountManager.ANONYMOUS_ID);
	
	public Handler(final WebSocket webSocket, final ClientHandshake handshake) {
		this.webSocket = webSocket;
		this.handshake = handshake;
		
		final GsonBuilder gb = new GsonBuilder();
		gb.registerTypeAdapter(byte[].class, new JsonSerializer<byte[]>() {
			@Override
			public JsonElement serialize(final byte[] src, final Type typeOfSrc, final JsonSerializationContext context) {
				return new JsonPrimitive(Base64.encodeBytes(src));
			}
		});
		gb.registerTypeAdapter(byte[].class, new JsonDeserializer<byte[]>() {
			@Override
			public byte[] deserialize(final JsonElement json, final Type typeOfT, final JsonDeserializationContext context) throws JsonParseException {
				try {
					return Base64.decode(json.getAsString());
				} catch (final IOException e) {
					e.printStackTrace();
					return null;
				}
			}
		});
		this.gson = gb.create();
	}
	
	public WebSocket getWebSocket() {
		return webSocket;
	}
	
	public ClientHandshake getHandshake() {
		return handshake;
	}
	
	public void send(final Object obj) {
		webSocket.send(gson.toJson(obj));
	}
	
	public FileSystem getFileSystem() {
		return fileSystem;
	}
	
	public Account getAccount() {
		return account;
	}
	
	public void setAccount(final Account v) {
		account = v;
	}
	
	public void handle(final String message) {
		final Packet packet = gson.fromJson(message, Packet.class);
		if (packet.opcode == null) {
			final String m = "No packet opcode\n";
			final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.FAILURE, m);
			send(r);
		} else if (packet.opcode.equals("API")) {
			final APIPacket p = gson.fromJson(message, APIPacket.class);
			final API command = APITable.INSTANCE.get(p.command);
			if (command == null) {
				final String m = p.command + ": command not found\n";
				final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.FAILURE, m);
				send(r);
			} else {
				command.execute(this, p);
			}
		} else if (packet.opcode.equals("FileAPI")) {
			final FileAPIPacket p = gson.fromJson(message, FileAPIPacket.class);
			final FileAPI command = FileAPITable.INSTANCE.get(p.command);
			if (command == null) {
				final String m = p.command + ": command not found\n";
				final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.FAILURE, m);
				send(r);
			} else {
				command.execute(this, p);
			}
		} else if (packet.opcode.equals("Command")) {
			final CommandPacket p = gson.fromJson(message, CommandPacket.class);
			final Command command = CommandTable.INSTANCE.get(p.command);
			if (command == null) {
				final String m = p.command + ": command not found\n";
				final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.FAILURE, m);
				send(r);
			} else {
				command.execute(this, p);
			}
		} else {
			final String m = packet.opcode + ": unknown packet type\n";
			final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.FAILURE, m);
			webSocket.send(gson.toJson(r));
		}
	}
	
}
